@@ -9,13 +9,14 @@ require Exporter;
 @EXPORT = qw(dd ddx);
 @EXPORT_OK = qw(dump pp dumpf quote);
 
-$VERSION = "1.16";
+$VERSION = "1.18";
 $DEBUG = 0;
 
 use overload ();
-use vars qw(%seen %refcnt @dump @fixup %require $TRY_BASE64 @FILTERS);
+use vars qw(%seen %refcnt @dump @fixup %require $TRY_BASE64 @FILTERS $INDENT);
 
 $TRY_BASE64 = 50 unless defined $TRY_BASE64;
+$INDENT = "  " unless defined $INDENT;
 
 sub dump
 {
@@ -66,7 +67,7 @@ sub dump
 
     if (%refcnt || %require) {
 	$out .= ";\n";
-	$out =~ s/^/  /gm;  # indent
+	$out =~ s/^/$INDENT/gm;
 	$out = "do {\n$out}";
     }
 
@@ -191,7 +192,7 @@ sub _dump
 		my $v = "$rval";
 
 		my $mod = "";
-		if ($v =~ /^\(\?([msix-]+):([\x00-\xFF]*)\)\z/) {
+		if ($v =~ /^\(\?\^?([msix-]*):([\x00-\xFF]*)\)\z/) {
 		    $mod = $1;
 		    $v = $2;
 		    $mod =~ s/-.*//;
@@ -225,7 +226,7 @@ sub _dump
 	    if (!defined $$rval) {
 		$out = "undef";
 	    }
-	    elsif ($$rval =~ /^-?[1-9]\d{0,9}$/ || $$rval eq "0") {
+	    elsif ($$rval =~ /^-?[1-9]\d{0,9}\z/ || $$rval eq "0") {
 		$out = $$rval;
 	    }
 	    else {
@@ -347,20 +348,24 @@ sub _dump
 	    }
 	}
 	$out = "{$nl";
-	$out .= "  # $tied$nl" if $tied;
+	$out .= "$INDENT# $tied$nl" if $tied;
 	while (@keys) {
 	    my $key = shift @keys;
 	    my $val = shift @vals;
-	    my $pad = " " x ($klen_pad + 6);
-	    $val =~ s/\n/\n$pad/gm;
-	    $key = " $key" . " " x ($klen_pad - length($key)) if $nl;
-	    $out .= " $key => $val,$nl";
+	    my $vpad = $INDENT . (" " x ($klen_pad ? $klen_pad + 4 : 0));
+	    $val =~ s/\n/\n$vpad/gm;
+	    my $kpad = $nl ? $INDENT : " ";
+	    $key .= " " x ($klen_pad - length($key)) if $nl;
+	    $out .= "$kpad$key => $val,$nl";
 	}
 	$out =~ s/,$/ / unless $nl;
 	$out .= "}";
     }
     elsif ($type eq "CODE") {
-	$out = 'sub { "???" }';
+	$out = 'sub { ... }';
+    }
+    elsif ($type eq "VSTRING") {
+        $out = sprintf +($ref ? '\v%vd' : 'v%vd'), $$rval;
     }
     else {
 	warn "Can't handle $type data";
@@ -429,11 +434,39 @@ sub format_list
     my $paren = shift;
     my $comment = shift;
     my $indent_lim = $paren ? 0 : 1;
+    if (@_ > 3) {
+	# can we use range operator to shorten the list?
+	my $i = 0;
+	while ($i < @_) {
+	    my $j = $i + 1;
+	    my $v = $_[$i];
+	    while ($j < @_) {
+		# XXX allow string increment too?
+		if ($v eq "0" || $v =~ /^-?[1-9]\d{0,9}\z/) {
+		    $v++;
+		}
+		elsif ($v =~ /^"([A-Za-z]{1,3}\d*)"\z/) {
+		    $v = $1;
+		    $v++;
+		    $v = qq("$v");
+		}
+		else {
+		    last;
+		}
+		last if $_[$j] ne $v;
+		$j++;
+	    }
+	    if ($j - $i > 3) {
+		splice(@_, $i, $j - $i, "$_[$i] .. $_[$j-1]");
+	    }
+	    $i++;
+	}
+    }
     my $tmp = "@_";
     if ($comment || (@_ > $indent_lim && (length($tmp) > 60 || $tmp =~ /\n/))) {
 	my @elem = @_;
-	for (@elem) { s/^/  /gm; }   # indent
-	return "\n" . ($comment ? "  # $comment\n" : "") .
+	for (@elem) { s/^/$INDENT/gm; }
+	return "\n" . ($comment ? "$INDENT# $comment\n" : "") .
                join(",\n", @elem, "");
     } else {
 	return join(", ", @_);
@@ -471,6 +504,7 @@ sub str {
       # 17 bytes (not counting any require statement needed).
       # But on the other hand, hex is much more readable.
       if ($TRY_BASE64 && length($_[0]) > $TRY_BASE64 &&
+	  (defined &utf8::is_utf8 && !utf8::is_utf8($_[0])) &&
 	  eval { require MIME::Base64 })
       {
 	  $require{"MIME::Base64"}++;
@@ -522,17 +556,17 @@ Data::Dump - Pretty printing of data structures
 
 =head1 SYNOPSIS
 
- use Data::Dump qw(dump ddx);
+ use Data::Dump qw(dump);
 
  $str = dump(@list);
  @copy_of_list = eval $str;
 
  # or use it for easy debug printout
- ddx localtime;
+ use Data::Dump; dd localtime;
 
 =head1 DESCRIPTION
 
-This module provide functions that takes a list of values as their
+This module provide a few functions that traverse their
 argument and produces a string as its result.  The string contains
 Perl code that, when C<eval>ed, produces a deep copy of the original
 arguments.
@@ -545,7 +579,7 @@ that is easy to read.  Example:
 
 Produces:
 
-    (1, [2, 3], { 4 => 5 })
+    "(1, [2, 3], { 4 => 5 })"
 
 If you dump just a little data, it is output on a single line. If
 you dump data that is more complex or there is a lot of it, line breaks
@@ -591,7 +625,8 @@ Returns a quoted version of the provided string.
 
 It differs from C<dump($string)> in that it will quote even numbers and
 not try to come up with clever expressions that might shorten the
-output.
+output.  If a non-scalar argument is provided then it's just stringified
+instead of traversed.
 
 =item dd( ... )
 
@@ -608,15 +643,41 @@ printouts of state within programs.
 
 =item dumpf( ..., \&filter )
 
-Short hand for the dump_filtered() function of L<Data::Dump::Filtered>.
+Short hand for calling the dump_filtered() function of L<Data::Dump::Filtered>.
+This works like dump(), but the last argument should be a filter callback
+function.  As objects are visited the filter callback is invoked and it
+can modify how the objects are dumped.
+
+=back
+
+=head1 CONFIGURATION
+
+There are a few global variables that can be set to modify the output
+generated by the dump functions.  It's wise to localize the setting of
+these.
+
+=over
+
+=item $Data::Dump::INDENT
+
+This holds the string that's used for indenting multiline data structures.
+It's default value is "  " (two spaces).  Set it to "" to suppress indentation.
+Setting it to "| " makes for nice visuals even if the dump output then fails to
+be valid Perl.
+
+=item $Data::Dump::TRY_BASE64
+
+How long must a binary string be before we try to use the base64 encoding
+for the dump output.  The default is 50.  Set it to 0 to disable base64 dumps.
 
 =back
 
 
 =head1 LIMITATIONS
 
-Code references will be displayed as simply 'sub { "???" }' when
-dumped. Thus, C<eval>ing them will not reproduce the original routine.
+Code references will be dumped as C<< sub { ... } >>. Thus, C<eval>ing them will
+not reproduce the original routine.  The C<...>-operator used will also require
+perl-5.12 or better to be evaled.
 
 If you forget to explicitly import the C<dump> function, your code will
 core dump. That's because you just called the builtin C<dump> function
@@ -630,16 +691,17 @@ in-most-cases-excellent C<Data::Dumper>.  Basic ideas and some code
 are shared with Sarathy's module.
 
 The C<Data::Dump> module provides a much simpler interface than
-C<Data::Dumper>.  No OO interface is available and there are no
-configuration options to worry about (yet :-).  The other benefit is
+C<Data::Dumper>.  No OO interface is available and there are fewer
+configuration options to worry about.  The other benefit is
 that the dump produced does not try to set any variables.  It only
 returns what is needed to produce a copy of the arguments.  This means
-that C<dump("foo")> simply returns C<"foo">, and C<dump(1..5)> simply
-returns C<(1, 2, 3, 4, 5)>.
+that C<dump("foo")> simply returns C<'"foo"'>, and C<dump(1..3)> simply
+returns C<'(1, 2, 3)'>.
 
 =head1 SEE ALSO
 
-L<Data::Dumper>, L<Storable>
+L<Data::Dump::Filtered>, L<Data::Dump::Trace>, L<Data::Dumper>, L<JSON>,
+L<Storable>
 
 =head1 AUTHORS
 
